###############################################################################
##
## Microsoft NGWS Profiler Sample (ProfilerGCP) Makefile
##
###############################################################################

###############################################################################
##
## !!! EDIT THESE MACROS TO CORRECTLY LOCATE NGWS/WIN32 HEADERS/LIBS !!!
##
## NGWS_LIB_DIR, directory where NGWS libs live
## NGWS_INC_DIR, directory where NGWS headers live 
##
## WIN32_LIB_DIR, directory where Win32 libs live
## WIN32_INC_DIR, directory where Win32 headers live
## 
## E.G.
##
## WIN32_LIB_DIR= d:\progra~1\micros~1\vc98\lib
## WIN32_INC_DIR= d:\progra~1\micros~1\vc98\include
##
###############################################################################
NGWS_LIB_DIR= d:\progra~1\NGWSSDK\lib
NGWS_INC_DIR= d:\progra~1\NGWSSDK\include

WIN32_LIB_DIR= d:\progra~1\micros~1\vc98\lib
WIN32_INC_DIR= d:\progra~1\micros~1\vc98\include


###############################################################################
##
## Building options for nmake command
##
###############################################################################
!MESSAGE 
!MESSAGE NMAKE /f "ProfilerGCP.mak" 
!MESSAGE 


###############################################################################
##
## General
##
###############################################################################
!IF "$(OS)" == "Windows_NT"
NULL=
!ELSE 
NULL=nul
!ENDIF 

SOURCE=.

OUTDIR= $(SOURCE)\Debug
INTDIR= $(SOURCE)\Debug
INCLUDE= .;$(NGWS_INC_DIR);$(WIN32_INC_DIR)


###############################################################################
##
## Tools (compiler, linker, etc)
##
###############################################################################
_LINKER= link.exe
_COMPILER= cl.exe


###############################################################################
##
## Win32 Libs
##
###############################################################################
WIN32_SDK_LIBS= \
    "$(WIN32_LIB_DIR)\uuid.lib"		  \
    "$(WIN32_LIB_DIR)\ole32.lib"      \
    "$(WIN32_LIB_DIR)\libcpd.lib"     \
    "$(WIN32_LIB_DIR)\user32.lib"     \
    "$(WIN32_LIB_DIR)\oleaut32.lib"   \
    "$(WIN32_LIB_DIR)\advapi32.lib"   \
    "$(WIN32_LIB_DIR)\kernel32.lib"   


###############################################################################
##
## COR Libs
##
###############################################################################
NGWS_LIBS= \
    "$(NGWS_LIB_DIR)\corguids.lib"
    
    
###############################################################################
##
## Debugger Shell obj files
##
###############################################################################
OBJS= \
	"$(INTDIR)\RegUtil.obj" 		\
	"$(INTDIR)\ProfilerBase.obj" 	\
	"$(INTDIR)\classfactory.obj" 	\
	"$(INTDIR)\ProfilerHelper.obj" 	\
	"$(INTDIR)\ProfilerCallback.obj" 


###############################################################################
##
## MACROS
##
###############################################################################
_MACROS= \
    /D "_WIN32"                     \
    /D "_DEBUG"                     \
    /D "UNICODE"                    \
    /D "_UNICODE"                   \
    /D "WIN32_LEAN_AND_MEAN"        \
	/D "PROFILERILC_EXPORTS"


###############################################################################
##
## FLAGS
##
###############################################################################
_CFLAGS= \
   $(_MACROS) /nologo /Gz /MTd /W3 /Zi /Od /Gy \
   /YX /Fo"$(INTDIR)\\" /Fd"$(INTDIR)\\" /FD /c 


_LFLAGS= \
    /nologo /dll /def:"$(SOURCE)\ProfilerGCP.def" /incremental:no \
    /machine:I386 /out:"$(OUTDIR)\ProfilerGCP.dll"


###############################################################################
##
## Build Profiler dll
##
###############################################################################
ALL : "$(OUTDIR)\ProfilerGCP.dll"


###############################################################################
##
## Output Directory
##
###############################################################################
"$(OUTDIR)" :
    if not exist "$(OUTDIR)/$(NULL)" mkdir "$(OUTDIR)"


###############################################################################
##
## Rules
##
###############################################################################
.cpp{$(INTDIR)}.obj::
   $(_COMPILER) @<<
   $(_CFLAGS) $< 
<<

"$(OUTDIR)\ProfilerGCP.dll": "$(OUTDIR)" $(OBJS)
    $(_LINKER) @<<
  $(WIN32_SDK_LIBS) $(NGWS_LIBS) $(_LFLAGS) $(OBJS)
<<


###############################################################################
##
## Clean up
##
###############################################################################
CLEAN :
	-@erase "$(INTDIR)\vc60.idb"
	-@erase "$(INTDIR)\vc60.pdb"
	-@erase "$(INTDIR)\RegUtil.obj"
	-@erase "$(OUTDIR)\ProfilerGCP.dll"
	-@erase "$(OUTDIR)\ProfilerGCP.exp"
	-@erase "$(OUTDIR)\ProfilerGCP.ilk"
	-@erase "$(OUTDIR)\ProfilerGCP.lib"
	-@erase "$(OUTDIR)\ProfilerGCP.pdb"
	-@erase "$(INTDIR)\classfactory.obj"
	-@erase "$(INTDIR)\ProfilerBase.obj"
	-@erase "$(INTDIR)\ProfilerHelper.obj"
	-@erase "$(INTDIR)\ProfilerCallback.obj"


###############################################################################
##
## Profiler Sources
##
###############################################################################
"$(INTDIR)\RegUtil.obj":            $(SOURCE)\RegUtil.cpp        	"$(INTDIR)"
"$(INTDIR)\ClassFactory.obj":       $(SOURCE)\ClassFactory.cpp   	"$(INTDIR)"
"$(INTDIR)\ProfilerBase.obj":       $(SOURCE)\ProfilerBase.cpp   	"$(INTDIR)"
"$(INTDIR)\ProfilerHelper.obj":     $(SOURCE)\ProfilerHelper.cpp    "$(INTDIR)"
"$(INTDIR)\ProfilerCallback.obj":   $(SOURCE)\ProfilerCallback.cpp  "$(INTDIR)"






